@@ -10,13 +10,14 @@ use CPAN::Meta;
 use Exporter qw{ import };
 use ExtUtils::Manifest ();
 use File::Find ();
+use File::Glob ();
 use Module::Extract::Use;
 use Module::CoreList;
 use Module::Metadata;
 use Scalar::Util ();
 use Test::More 0.88;
 
-our $VERSION = '0.000_916';
+our $VERSION = '0.000_918';
 
 our @EXPORT_OK = qw{ all_prereq_ok file_prereq_ok prereq_ok };
 our %EXPORT_TAGS = (
@@ -40,11 +41,11 @@ use constant REF_ARRAY	=> ref [];
 sub new {
     my ( $class, %arg ) = @_;
 
+    $arg{file_error} //= 'Failed to analyze %f: %e';
+    $arg{name} //= 'Prereq test: %f uses %m';
     # NOTE that {path_type} is unsupported, and may change or be
     # retracted without warning. I thought I needed it to support
     # argument {prune}, which is itself experimental.
-    $arg{file_error} //= 'Failed to analyze %f: %e';
-    $arg{name} //= 'Prereq test: %f uses %m';
     $arg{path_type} //= DEFAULT_PATH_TYPE;
     $arg{per_file_note} //= '%f';
     $arg{perl_version} //= 'none';
@@ -56,17 +57,22 @@ sub new {
 	    MYMETA.json MYMETA.yml META.json META.yml } ],
 	prune	=> [],
 	uses	=> [],
+	verbose	=> (
+	    scalar grep { -d } qw{ .bzr .cdv .git .hg .svn CVS } ) ? 1 : 0,
     };
     foreach my $name ( keys %{ $default } ) {
 	$arg{$name} //= $default->{$name};
 	my $code = __PACKAGE__->can( "__validate_$name" ) ||
 	    __PACKAGE__->can( '__validate_' . ref $default->{$name} ) ||
-	    sub { $_[0] };
+	    sub {};
 	$code->( $name, \%arg );
     }
 
     my $core_modules;
     {
+	# %Module::CoreList::version is public, so I figured the easiest
+	# implementation of the 'special' Perl versions was to just hack
+	# them into it.
 	local $Module::CoreList::version{none} = {};
 	local $Module::CoreList::version{this} =
 	    $Module::CoreList::version{$]};
@@ -90,6 +96,10 @@ sub new {
 
     # The above is pretty much verbatim from the CPAN::Meta synopsis
 
+    # NOTE that if we actually need the Perl version, we need to nab it
+    # before here.
+    delete $requires{perl};
+
     my $provides = _provides();
 
     my %has = map { $_ => 1 }
@@ -98,24 +108,23 @@ sub new {
 	keys %{ $provides },
 	keys %requires,
 	;
-    delete $has{perl};
 
-    if ( my @dup = grep { $requires{$_} } @{ $arg{accept} } ) {
-	diag "The following @{[
-	    @dup == 1 ? 'module appears' : 'modules appear'
-	    ]} in both the prerequisites and\nthe 'accept' argument: ",
-	    join ', ', @dup;
+    $arg{uses} = { map { $_ => 1 } @{ $arg{uses} } };
+
+    if ( $arg{verbose} ) {
+	my @dup;
+	@dup = grep { $requires{$_} } @{ $arg{accept} }
+	    and diag "The following @{[
+		@dup == 1 ? 'module appears' : 'modules appear'
+		]} in both the prerequisites and\nthe 'accept' argument: ",
+		join ', ', sort @dup;
+	@dup = grep { $arg{uses}{$_} } @{ $arg{accept} }
+	    and diag "The following @{[
+		@dup == 1 ? 'module appears' : 'modules appear'
+		]} in both the 'accept' argument and\nthe 'uses' argument: ",
+		join ', ', sort @dup;
     }
 
-    # FIXME the following is because we don't scan Makefile.PL and/or
-    # Build.PL
-    -e 'Makefile'
-	and $has{'ExtUtils::MakeMaker'}
-	and push @{ $arg{uses} }, 'ExtUtils::MakeMaker';
-    -e 'Build'
-	and $has{'Module::Build'}
-	and push @{ $arg{uses} }, 'Module::Build';
-
     delete $arg{accept};
     delete $arg{_meta_file};
     delete $arg{path_type};
@@ -133,7 +142,9 @@ sub new {
 	prune		=> delete $arg{prune},
 	# provides	=> $provides,
 	skip_name	=> delete $arg{skip_name},
-	uses		=> { map { $_ => 1 } perl => @{ delete $arg{uses} } },
+	uses		=> delete $arg{uses},
+	verbose		=> delete $arg{verbose},
+	_both_tools	=> ( -e 'Makefile.PL' && -e 'Build.PL' ),
 	_normalize_path	=> delete $arg{_normalize_path},
 	_requires	=> \%requires,
     }, ref $class || $class;
@@ -148,8 +159,13 @@ sub new {
 
 sub all_prereq_ok {
     my ( $self, @file ) = _unpack_args( @_ );
-    @file
-	or @file = grep { -d } qw{ blib/arch blib/lib blib/script t };
+
+    unless( @file ) {
+	@file = (
+	    ( grep { -d } qw{ blib/arch blib/lib blib/script t } ),
+	    File::Glob::bsd_glob( '*.PL' ),
+	);
+    }
 
     my $need_skip = 1;
     my $ok = 1;
@@ -193,6 +209,8 @@ sub all_prereqs_used {
     state $TEST = Test::More->builder();
     local $Test::Builder::Level = _nest_depth();
 
+    $TEST->note( '' );
+
     my @unused = sort
 	grep { ! $self->{uses}{$_} && ! $self->{_requires}{$_}{file} }
 	keys %{ $self->{_requires} };
@@ -200,6 +218,17 @@ sub all_prereqs_used {
 	or $TEST->diag( "The following @{[
 	    @unused == 1 ? 'prerequisite is' : 'prerequisites are'
 	    ]} unused: ", join ', ', @unused );
+
+    if ( $self->{verbose} and
+	my @dup = grep { $self->{_requires}{$_}{file} && $self->{uses}{$_} }
+	keys %{ $self->{_requires} }
+    ) {
+	$TEST->diag( "The following @{[
+	    @dup == 1 ? 'module appears' : 'modules appear'
+	    ]} in both 'use' statements and\nthe 'uses' argument: ",
+	    join ', ', sort @dup );
+    }
+
     return $rslt;
 }
 
@@ -213,6 +242,7 @@ sub file_prereq_ok {
     # (localized) to get tests reported relative to the correct file and
     # line, rather than setting the 'level' attribute.
     state $TEST = Test::More->builder();
+    local $Test::Builder::Level = _nest_depth();
 
     if ( $self->{per_file_note} ne '' ) {
 	# We are not interested in the actual test number, but we need
@@ -240,7 +270,6 @@ sub file_prereq_ok {
 
     my $modules = $extor->get_modules_with_details( $file );
     if ( my $err = $extor->error() ) {
-	local $Test::Builder::Level = _nest_depth();
 	$TEST->ok( 0,
 	    _format(
 		$self->{file_error},
@@ -258,7 +287,6 @@ sub file_prereq_ok {
 	sort { $a->{module} cmp $b->{module} }
 	@{ $modules }
     ) {
-	local $Test::Builder::Level = _nest_depth();
 	my $module = $usage->{module};
 
 	# The following is needed because Module::Extract::Use tries too
@@ -276,9 +304,22 @@ sub file_prereq_ok {
 	$self->{_requires}{$module}
 	    and push @{ $self->{_requires}{$module}{file} ||= [] }, $file;
 
+	state $toolchain = {
+	    'Makefile.PL'	=> {
+		'ExtUtils::MakeMaker'	=> 1,
+		'inc::Module::Install'	=> 1,
+	    },
+	    'Build.PL'		=> {
+		'Module::Build'		=> 1,
+		'Module::Build::Tiny'	=> 1,
+	    },
+	};
+
 	$need_skip = 0;
 	$TEST->ok(
-	    $self->{has}{$module} || 0,
+	    $self->{has}{$module} ||
+	    $self->{_both_tools} && $toolchain->{$file}{$module} ||
+	    0,
 	    _format(
 		$self->{name},
 		{
@@ -521,9 +562,11 @@ This method accepts the following arguments as name/value pairs:
 
 This argument is the name of a module, or a reference to an array of
 module names. These modules will be passed even if they are not listed
-as prerequisites. However, if any module listed here is found in the
-prerequisites, a diagnostic will be generated when the object is
-instantiated.
+as prerequisites.
+
+If the L<verbose|/verbose> attribute is true, a diagnostic will be
+emitted for any modules listed here which also appear in the
+prerequisites or the L<uses|/uses> attribute.
 
 The default is C<[]>, that is, a reference to an empty array.
 
@@ -641,8 +684,23 @@ This argument is the name of a module, or a reference to an array of
 module names. The L<all_prereqs_used()|/all_prereqs_used> test will
 count these as having been used, even if no use of them is found.
 
+If the L<verbose|/verbose> attribute is true, a diagnostic will be
+emitted for any modules listed here which also appear in the
+L<accept|/accept> attribute.
+
 The default is C<[]>, that is, a reference to an empty array.
 
+=item verbose
+
+This Boolean argument specifies whether diagnostics are generated on
+redundant C<accept> and C<uses> specifications.
+
+On the presumption that these are more likely to be of use to a module
+author than a module user, the default is true if and only if at least
+one of the following directories exists at the distribution's top level:
+
+    .bzr .cdv .git .hg .svn CVS
+
 =back
 
 The use of arguments other than the above will result in an exception.
@@ -707,6 +765,10 @@ to test all relevant files in the distribution using
 L<all_prereq_ok()|/all_prereq_ok> or
 L<file_prereq_ok()|/file_prereq_ok>.
 
+If the L<verbose|/verbose> attribute is true, a diagnostic will be
+emitted for any modules listed in the L<uses|/uses> attribute which were
+actually used by the code being tested.
+
 =head2 file_prereq_ok
 
  $tpm->file_prereq_ok( $file_name );
@@ -775,6 +837,15 @@ what modules are provided by the distribution. If this is absent, it
 uses L<Module::Metadata|Module::Metadata> to determine provided modules
 directly from F<blib/lib/>.
 
+It is a personal crotchet of mine that if a distribution provides both
+F<Makefile.PL> and F<Build.PL>, that neither
+L<ExtUtils::MakeMaker|ExtUtils::MakeMaker> nor
+L<Module::Build|Module::Build> should be a prerequisite. The actual
+situation is that one or the other is required, but both are not. But
+the Perl dependency system appears to have no way to represent this. So
+in the presence of both F<Makefile.PL> and F<Build.PL>, I have indulged
+my whim.
+
 =head1 SEE ALSO
 
 L<Test::Dependencies|Test::Dependencies> by Erik Huelsmann, which
